// -----------------------------------------------------------------------------
// File name: delay.c
// Author: Roberto Fonseca Iannini
// E-mail: beto1684@yahoo.com.br
// -----------------------------------------------------------------------------
// REVISION HISTORY
// VER     DATE                   DESCRIPTION
// 0.1  30 dec 2005   Delay routines precisely tuned for HSPLL = 40MHz
//
// -----------------------------------------------------------------------------
// KEYWORDS: microsecond, milisecond, second
//           delay, time, nop
// -----------------------------------------------------------------------------
// PURPOSE: delay routines
//
// -----------------------------------------------------------------------------
// PARAMETERS:
// see delay.h
// -----------------------------------------------------------------------------
// REUSE ISSUES
// Oscillator:  xtal @ 10MHz (x4 PLL = 40MHz)
//              i.e, one instruction (4 cycles each) = 1/10MHz = 0.1uS
// Peripherals: none
// Processor:   PIC18F452
// Other:       Register names differ from 16F877
//
// -----------------------------------------------------------------------------


//#include "delay.h"


//==============================================================================
// Microseconds - 0 to 255 uS
//                                Note: From 0 to 3, nothing is actually
//                                      executed, but the it spend 1.4uS
//                                +---------+---------------------------+------+
//                                | setting |        actual delay       |error |
//                                |         | w/ period counter attached|      |
//                                |  0 - 3  |           1.4uS           | ~40% |
//                                |    5    |           6.7uS           |  33% |
//                                |    8    |           9.7uS           |  20% |
//                                |   10    |          11.7uS           |  17% |
//                                |   20    |          21.7uS           |   8% |
//                                |   30    |          31.7uS           |   5% |
//                                |   80    |          81.7uS           |   2% |
//                                |  100    |         101.6uS           |   2% |
//                                |  150    |         151.6uS           |   1% |
//              IMPORTANT ---->   |  250    |         251.6uS           | 0.6% |
//                                |  255    |         256.6uS           | 0.6% |
//                                +---------+----------------------------------+
//==============================================================================
void
delay_us(unsigned char dly_us)
{
  if (dly_us > 3)
   {
    while (dly_us)
     {
      asm("nop"); asm("nop"); asm("nop"); asm("nop");      // 0.1uS each "nop"
      dly_us--;
     }
   };
}



//==============================================================================
// Miliseconds - 0 to 255 mS
//                                +---------+---------------------------+------+
//                                | setting |        actual delay       |error |
//                                |         | w/ period counter attached|      |
//                                |    1    |          1007uS           | 0.7% |
//                                |    2    |          2012uS           | 0.6% |
//                                |   10    |         10058uS           | 0.6% |
//                                |  100    |         100.5mS           | 0.5% |
//                                |  200    |         201.0mS           | 0.5% |
//              IMPORTANT ---->   |  250    |         251.4mS           | 0.5% |
//                                |  255    |         256.2mS           | 0.5% |
//                                +---------+----------------------------------+
//==============================================================================
void
delay_ms(unsigned char dly_ms)
{
  while (dly_ms)
   {
    delay_us(250);
    delay_us(250);
    delay_us(250);
    delay_us(250);  // 1/4ms * 4 = 1ms
    dly_ms--;
   }
}



//==============================================================================
// Seconds - 0 to 255 S
//                                +---------+---------------------------+------+
//                                | setting |        actual delay       |error |
//                                |         | w/ period counter attached|      |
//                                |    1    |          1006mS           | 0.6% |
//                                |    2    |          2011mS           | 0.6% |
//                                |    3    |          3017mS           | 0.6% |
//                                | others  | counter overflow. Estimate  0.6% |
//                                +---------+----------------------------------+
//==============================================================================
void
delay_s(unsigned char dly_s)
{
  while (dly_s)
   {
    delay_ms(250);
    delay_ms(250);
    delay_ms(250);
    delay_ms(250);  // 1/4s * 4 = 1s
    dly_s--;
   }
}
