// -----------------------------------------------------------------------------
// File name: func.c
// Author: Roberto Fonseca Iannini
// E-mail: beto1684@yahoo.com.br
// -----------------------------------------------------------------------------
// REVISION HISTORY
// VER     DATE                   DESCRIPTION
// 0.2  17 jan 2006   Lots of improvements! LCD work fine in graph and text
//                    modes.
// 0.1  30 dec 2005   Graphical LCD initialization ok.
//
// -----------------------------------------------------------------------------
// KEYWORDS: glcd, keyboard scan, text_mode graph_mode
// -----------------------------------------------------------------------------
// PURPOSE: control the graphical LCD model LM238-X (chipset HD61830), a 6 key
// keyboard, some menus and still have 90% of time processing free
//
// -----------------------------------------------------------------------------
// PARAMETERS:
// see ".h" files
// -----------------------------------------------------------------------------
// REUSE ISSUES
// Oscillator:  xtal @ 10MHz (x4 PLL)
// Peripherals: USART, I/O ports, timer1
// Processor:   PIC18F452
// Other:       Register names differ from 16F877
//
// -----------------------------------------------------------------------------





//#include "func.h"
//==============================================================================
// DEFINES
//==============================================================================
#ifndef LCD_PORT_H
#define LCD_PORT_H

#define BAUD 115200        // RS-232 baud rate
#define FOSC 48000000L     // 12MHz HSPLL x4 (or "32000000L" for 8MHz x4)

#define NINE 0             // Use 9bit communication? FALSE=8bit
#define OUTPUT 0
#define INPUT 1

#define DIVIDER ((int)(FOSC/(16UL * BAUD) -1))
#define HIGH_SPEED 1

#if NINE == 1
  #define NINE_BITS 0x40
#else
  #define NINE_BITS 0
#endif

#if HIGH_SPEED == 1
  #define SPEED 0x4
#else
  #define SPEED 0
#endif

#define FREQ_MULT (FOSC)/(4000000L)

#define L_RED    RA0       // Led red   (A0)
#define L_GREEN  RA1       // Led green (A1)
#define KBO_A    RB3       // Keyboard scan out line 0
#define KBO_B    RB4       // Keyboard scan out line 1
#define KBO_C    RB5       // Keyboard scan out line 2
#define KBI_A    RB6       // Keyboard scan in  line 0
#define KBI_B    RB7       // Keyboard scan in  line 1
#define LCD_CS_B RC0       // LCD CS borrow
#define LCD_RS   RE0       // LCD reset
#define LCD_RW   RE1       // LCD Read(1)/write(0)
#define LCD_EN   RE2       // LCD enable
#define BUS      PORTD     // General purpose I/O data bus (used by the LCD too)
#define BUS_DIR  TRISD     // General purpose I/O data bus direction register
#define BUS_IN   0xFF      // BUS as in
#define BUS_OUT  0x00      // BUS as out

#define KB_OUT_A RB3       // saida de scan para teclado
#define KB_OUT_B RB4       // saida de scan para teclado
#define KB_OUT_C RB5       // saida de scan para teclado
#define KB_IN_A  RB6       // entrada de scan para teclado
#define KB_IN_B  RB7       // entrada de scan para teclado

#define K_RIGHT (KB_OUT_A && KB_IN_A)
#define K_UP    (KB_OUT_A && KB_IN_B)
#define K_DOWN  (KB_OUT_B && KB_IN_A)
#define K_LEFT  (KB_OUT_B && KB_IN_B)
#define K_ENTER (KB_OUT_C && KB_IN_A)
#define K_CLR   (KB_OUT_C && KB_IN_B)

#define RIGHT_P 6          // -+
#define UP_P    5          //  |
#define DOWN_P  4          //  |
#define LEFT_P  3          //  | keyboard pressed values
#define ENTER_P 2          //  |
#define CLR_P   1          //  |
#define NONE_P  0          // -+

#define INIT_ST     0      // -+
#define IDLE_ST     1      //  |
#define TXT_INIT_ST 2      //  | FSM states
#define TXT_ST      3      //  |
#define G_INIT_ST   4      //  |
#define G_ST        5      // -+


//--- TIMER0 ---
/*
#define OPTION_TMR0 0b10000111  // pullUp off, tmr0-> presc 256
#define TRISA_CONF 0b11011111
#define TRISB_CONF 0b11111111
#define TRISC_CONF 0b11011011
#define TRISD_CONF 0b11111111
#define TRISE_CONF 0b00000000
*/


//==============================================================================
// Bitmap en memoire i2c 24C256
//           pageMem    // Description           pageMemReel = pageMem>>1
//==============================================================================
#define bmp0  0     // seb scope
#define bmp1  4     // winamp
#define bmp2  8     // symbol
#define bmp3  20      // USA
#define bmp4  16      // go wild
#define bmp5  12      // graph data
#define bmp6  24      // Alarme (voiture)
#define bmp7  28      // G12032
#define bmp8  32      // lextronic
#define bmp9  36      // V
#define bmp10 40      // V
#define bmp11 44      // V
#define bmp12 48      // V
#define bmp13 52      // V
#define bmp14 56      // V
#define bmp15 60      // V
#define bmp16 64      // menu Principal
#define bmp17 68      // menu Scope
#define bmp18 72      // menu Dat8
#define bmp19 76      // menu Tra2  (transfert Bootloader)
#define bmp20 80      // menuCal2 : ampoule off
#define bmp21 84      // menuCal2 : ampoule on
#define bmp22 88      // menuSco3 : menu param scope
#define bmp23 92      // menuI2C  : transfert bitmap
#define bmp24 96      // menuAide : copyright
#define bmp25 100   // V
#define bmp26 104   // V
#define bmp27 108   // V
#define bmp28 112   // V
#define bmp29 116   // V
#define bmp30 120   // V
#define bmp31 124   // V

#endif


//==============================================================================
// Global variables
//==============================================================================
unsigned char bufferRX[64];
unsigned char indexRX = 0;
static bit serialRX;
unsigned char kb_value, curr_st, next_st;
unsigned char print;
unsigned char h_plot, v_plot;


//==============================================================================
// Keypad read
//==============================================================================
unsigned char key_read(void)
{
 unsigned char value = NONE_P;
 KB_OUT_A = 1;
 KB_OUT_B = 0;
 KB_OUT_C = 0;
 kb_debounce();
   if (KB_IN_A)
   {
    value = RIGHT_P;
    return(value);
   }
   else if (KB_IN_B)
   {
    value = UP_P;
    return(value);
   };
 KB_OUT_A = 0;
 KB_OUT_B = 1;
 KB_OUT_C = 0;
 kb_debounce();
   if (KB_IN_A)
   {
    value = DOWN_P;
    return(value);
   }
   else if (KB_IN_B)
   {
    value = LEFT_P;
    return(value);
   };
 KB_OUT_A = 0;
 KB_OUT_B = 0;
 KB_OUT_C = 1;
 kb_debounce();
   if (KB_IN_A)
   {
    value = ENTER_P;
    return(value);
   }
   else if (KB_IN_B)
   {
    value = CLR_P;
    return(value);
   };
 return(value);
}

//==============================================================================
// Keyboard debounce
//==============================================================================
void
kb_debounce(void)
 {
 unsigned int time_d = 1000;
 while (time_d > 0)
  {
   time_d--;
  };
 }


//==============================================================================
// FSM states
//==============================================================================

//--------
// init_st
//--------
void
fsm_init_st(void)
 {
  //CCP1IE = 1;                  // CCPM1 int enable
  PEIE = 1;                      // Peripheral interrupt enable (timer. CCP)
  GIE = 1;                       // todas as int habilitadas (GLOBAL)
  ADCON1 = 0x06;                 // <RA7:RA0> as digital I/O  (18F452 - pg 182)
                                 // when using 16F877 (pg112):
                                 // <RA5:RA0> and <RE2:RE0> as digital I/O
  TRISA = 0b11111100;            // <RA1:RA0> as output. <7:2> as input
  TRISB = 0b11000111;            // RB0   (no use yet) in
                                 // RB1   (no use yet) in
                                 // RB2   (no use yet) in
                                 // RB3   ("KB_OUT_A") out
                                 // RB4   ("KB_OUT_B") out
                                 // RB5   ("KB_OUT_C") out
                                 // RB6   ("KB_IN_A")  in
                                 // RB7   ("KB_IN_B")  in
  RBPU   = 0;                    // PORTB Pull-up Enabled
  TRISC = 0xFE;                  // RC0   ("LCD_CS_B") out
                                 // <RC7:RC1> (no use yet)in
  BUS_DIR = BUS_IN;              // PORTD ("BUS")         in (dinamically changes)
  TRISE = 0x00;                  // RE0   ("LCD_RS")      out
                                 // RE1   ("LCD_RW")      out
                                 // RE2   ("LCD_EN")      out
                                 // Attention! BIT4 control PORTD as slave PP
                                 // see datasheet pg 35 (16F877)/pg 95 (18F452)
  RBPU = 0;                      // PORTB pull-up on
  INTEDG0 = 0;                   // negedge interrupt for RB0 (not used yet)
  // TIMER 0 *8bits* (used for keyboard scan)
  TMR0ON = 1;                    // Timer0 On/Off Control bit
  T08BIT = 1;                    // Timer0 8-bit mode
  T0CS = 0;                      // fonte de clock interna (Fosc/4)
  PSA = 0;                       // prescaler designado para o timer0
  T0PS0 = 1;                     // -+
  T0PS1 = 1;                     //  |prescaler timer0 = 256
  T0PS2 = 1;                     // -+

  // USART
  TXEN = 1;                      // escreve 1 em TXIF automaticamente
  SYNC = 0;                      // modo async da USART
  BRGH = 0;                      // modo low speed da USART
  SPEN = 1;                      // RS-232 enable (RC7->Rx e RC6->Tx)
  CREN = 1;                      // recepcao continua. USART enable
  SPBRG = 23;                    // =23: 9600bps  (0% de erro @ 14.7456MHz)
                                 // =11: 19200bps (0% de erro @ 14.7456MHz)
                                 // =3:  57600bps (0% de erro @ 14.7456MHz)
  // TIMER 1 *16bits* (usado para CLK_MOTOR)
  T1CON = 0b00110000;            // T1CKPS1 = 1;  prescaler 1:8
                                 // T1CKPS0 = 1;  prescaler 1:8
                                 // T1OSCEN = 0;  osc enabled
                                 // T1SYNC  = 0;  sync
                                 // TMR1CS  = 0;  internal clock
                                 // TMR1ON  = 0;  TMR1 stoped
  // CAPTURE COMPARE MODULE
  CCP1CON = 0b00001010;          // CCP1 no modo compare: Gera uma
                                 // int (CCP1IF) no match. Ver pag. 55

  // SOME BASIC ACTIONS
  LCD_CS_B = 1;                  // Disable LCD



  RBIE   = 0;                    // interrupt on change
  TMR0IE   = 0;                  // interrupcao de timer0 p/ gerar teclado
  CCP1IE = 0;                    // CCPM1 int enable
  PEIE   = 0;                    // Peripheral interrupt enable (timer. CCP)
  GIE    = 0;                    // todas as int habilitadas (GLOBAL)

  KB_OUT_A = 0;
  KB_OUT_B = 0;
  KB_OUT_C = 0;

  lcd_init_text();
  lcd_clear();
  L_GREEN = 0;
  L_RED   = 1;
  delay_ms(250);
    L_GREEN = 1;
  L_RED   = 0;
/*
  lcd_puts("       Tela para agradar a mae:         ");
  lcd_puts("________________________________________");
  lcd_puts("                                        ");
*/
  delay_ms(250);
  L_GREEN = 0;
  L_RED   = 0;
  //Interrupts ON
  //RBIE = 1;                    // interrupt on change
  TMR0IE = 0;                    // interrupcao de timer0 p/ gerar teclado
  INT0IE = 0;                    // INT0 External Interrupt Enable bit
  RBIE   = 0;                    // RB Port Change Interrupt Enable bit
 }

//------------------
// fsm_graph_demo_st
//------------------
void
fsm_graph_demo_st(void)
 {
/*
  // Centered "X"
  lcd_line(100,108,140,20);
     delay_s(1);
  lcd_line(100,20,140,108);
     delay_s(1);
  // Centered "+" (full screen)
  lcd_line(120,0,120,127);
     delay_ms(100);
  lcd_line(0,63,239,63);
     delay_ms(100);
  // Four ramps from orign to end y=128
  lcd_line(0,0,239,127);
     delay_ms(250);
  lcd_line(0,0,179,127);
     delay_ms(250);
  lcd_line(0,0,119,127);
     delay_ms(250);
  lcd_line(0,0,59,127);
     delay_ms(250);
  // Four ramps from end y to x=0
  lcd_line(0,127,239,0);
     delay_ms(100);
  lcd_line(0,127,179,0);
     delay_ms(100);
  lcd_line(0,127,119,0);
     delay_ms(100);
  lcd_line(0,127,59,0);
     delay_ms(100);
  // Multiple circles (forms a cone)
  for (print = 1; print <40; print++)
   {
    delay_ms(10);
    lcd_circle(40+2*print,10+print,print);
   }
     delay_s(1);
  // One pixel ON
  lcd_set_plot(200,50,1);
     delay_ms(250);
  // Rectangle (wide screen)
  lcd_rect(0,0,239,9);
     delay_ms(250);
  // Static grid
  lcd_grid();
//   lcgellipse(35,35,50,35,1);  // lcgellipse(int xc,int yc,int a0,int b0,int c)
     delay_ms(250);
//   lcd_goto(63,70);            // no effect
  // Bitmap plot (128 x 64)
  lcd_bitmap_plot(119, 64, 128, 64, img3);
*/


/*
  lcd_graph_font = FNT68;
  //lcd_puts("!#$%&'()*+,-./0123456789:;<=>?",0,10);
    lcd_puts("{|}~",0,10);
  lcd_puts("Tela autista",10,18);
*/

  lcd_line(0,0,0,127);
  lcd_line(0,0,127,0);
  lcd_line(0,9,127,9);
  lcd_line(8,0,8,127);

  lcd_graph_font = FNT88;
  lcd_puts("LCD   do borogod!",0+8,1);
  lcd_puts("LCD   do borogod!",1+8,10);
  lcd_puts("LCD   do borogod!",2+8,10+8);
  lcd_puts("LCD   do borogod!",3+8,10+8+8);
  lcd_puts("LCD   do borogod!",4+8,10+8+8+8);
  lcd_puts("LCD   do borogod!",5+8,10+8+8+8+8);
  lcd_puts("LCD   do borogod!",6+8,10+8+8+8+8+8);
  lcd_puts("LCD   do borogod!",7+8,10+8+8+8+8+8+8);
  lcd_puts("!LCD !",180,10+8+8+8+8+8+8+8);

  lcd_graph_font = FNT88B;
  lcd_puts("",0+8+8+8,10+8+8+8+8+8+8+8+8);
  lcd_puts("",1+8+8+8,10+8+8+8+8+8+8+8+8+8);
  lcd_puts("",2+8+8+8,10+8+8+8+8+8+8+8+8+8+8);
  lcd_puts("",3+8+8+8,10+8+8+8+8+8+8+8+8+8+8+8);
  lcd_puts("",4+8+8+8,10+8+8+8+8+8+8+8+8+8+8+8+8);
  lcd_puts("!LCD !",180,10+8+8+8+8+8+8+8+8+8+8+8+8+8);
  lcd_puts("",6+8+8+8,10+8+8+8+8+8+8+8+8+8+8+8+8+8+8);

/*
  lcd_graph_font = FNT816;
  lcd_puts("",8,70);
  lcd_puts(" Tela autista",9,96);
*/
 }

// !"#$%&'()*+,-./0123456789:;<=>?@OPQRSTUVWXYZ[\]^_`opqrstuvwxyz{|}~
// 
