// -----------------------------------------------------------------------------
// File name: lm238.c
// Author: Roberto Fonseca Iannini
// E-mail: beto1684@yahoo.com.br
// -----------------------------------------------------------------------------
// REVISION HISTORY
// VER     DATE                   DESCRIPTION
// 0.3  18 fev 2006   The finite state machine (FSM) is working properly. For
//                     educational purposes, in the next version I will implement
//                     a bypass to allow the LCD to run in "demosntration mode".
// 0.2  17 jan 2006   Lots of improvements! LCD work fine in graph and text
//                    modes.
// 0.1  30 dec 2005   Graphical LCD initialization ok.
//
// -----------------------------------------------------------------------------
// KEYWORDS: glcd, keyboard scan, text_mode graph_mode
// -----------------------------------------------------------------------------
// PURPOSE: control the graphical LCD model LM238-X (chipset HD61830), a 6 key
// keyboard, some menus and still have 90% of time processing free
//
// -----------------------------------------------------------------------------
// PARAMETERS:
// see ".h" files
// -----------------------------------------------------------------------------
// REUSE ISSUES
// Oscillator:  xtal @ 10MHz (x4 PLL)
// Peripherals: USART, I/O ports, timer1
// Processor:   PIC18F452
// Other:       Register names differ from 16F877
//
// -----------------------------------------------------------------------------

// include images:
   #include "bmp\img1.h"
   #include "bmp\img2.h"
   #include "bmp\img3.h"
   #include "bmp\img4.h"
   //#include "img5.h"
   #include "bmp\img6.h"
   #include "bmp\img7.h"

// include graphical text fonts:
   #include "fonts\fnt68.h"
   #include "fonts\fnt88.h"
   #include "fonts\fnt88b.h"
   #include "fonts\fnt816.h"

// include files:
   #include "pic18.h"
   #include "delay.h"
   #include "func.h"
   #include "lcd.h"
   #include "delay.c"
   #include "func.c"
   #include "lcd.c"
   #include "stddef.h"

//==============================================================================
// FUSES
//==============================================================================
  __CONFIG(1, HSPLL & OSCSDIS);   //   HS w/PLL Enabled, Clk Freq = 4xFreq
//__CONFIG(1, HS & OSCSDIS);      //   for freq>8Mhz and <20Mhz, use XT
                                  //   instead of HS for <8Mhz
//__CONFIG(1, XT & OSCSDIS);      //   XT Enabled, Clk Freq = 1xFreq
                                  // OSCDIS (see "oscen" in the datasheet):
                                  //   Allows the system clock source to be
                                  //   switched from the main oscillator to an
                                  //   alternate low frequency clock source
                                  // Oscillator optitons:
                                  //   1. LP Low Power Crystal
                                  //   2. XT Crystal/Resonator
                                  //   3. HS High Speed Crystal/Resonator
                                  //   4. HS + PLL High Speed Crystal/Resonator
                                  //      with PLL enabled
                                  //   5. RC External Resistor/Capacitor
                                  //   6. RCIO External Resistor/Capacitor with
                                  //      I/O pin enabled
                                  //   7. EC External Clock
                                  //   8. ECIO External Clock with I/O pin
                                  //      enabled
  __CONFIG(2,BORDIS & WDTDIS);              // BORDIS=brown out reset disabled,
                                            // WDTDIS=watch dog timer disabled
  __CONFIG(4,DEBUGDIS & LVPDIS & STVRDIS);  // DEBUGDIS=disabled, LVPDIS=low
                                            // voltage programmin disabled
  __CONFIG(5,UNPROTECT);                    // unprotected
  __CONFIG(6,WRTEN);                        // flash write enabled
  __CONFIG(7,TRU);                          // all unprotected incl. boot block



//==============================================================================
// Global variables: see func.c
//==============================================================================


//==============================================================================
// main
//==============================================================================
main()
{
 print = 60;
 curr_st = INIT_ST;
 next_st = curr_st;
 while(1)
  {
   while ( KB_IN_A || KB_IN_B );            // get stuck while keyboard pressed
   curr_st = next_st;                       // (avoid auto-repeat)
   switch(curr_st)
    {
      case INIT_ST:
        {
          fsm_init_st();
          lcd_puts("        Roberto Fonseca Iannini         ",0,0);
          lcd_puts("________________________________________",0,1);
          lcd_puts("                                        ",0,2);
          lcd_puts("UP: text mode    ",10,5);
          lcd_puts("DOWN: graph mode   ",10,6);
          next_st = IDLE_ST;
          break;
        }
      case IDLE_ST:
        {
          kb_value=key_read();
          if      (kb_value == ENTER_P)
           {
            L_GREEN = 1;
            L_RED   = 1;
           }
          else if (kb_value == CLR_P)
           {
            L_GREEN = 0;
            L_RED   = 0;
           }
          else if (kb_value == RIGHT_P)
           {
            L_GREEN = !L_GREEN;
           }
          else if (kb_value == LEFT_P)
           {
            L_RED = !L_RED;
           }
          else if (kb_value == UP_P)
           {
            next_st = TXT_INIT_ST;
           }
          else if (kb_value == DOWN_P)
           {
            next_st = G_INIT_ST;
           };
          break;
        }
      case TXT_INIT_ST:
       {
         lcd_init_text();
         lcd_clear();
         //lcd_goto(5,5);
         lcd_puts("curr_st = TXT_ST ",10,10);
         next_st = TXT_ST;
         break;
       }
      case G_INIT_ST:
       {
         lcd_init_graphic();
         next_st = G_ST;
         break;
       }
      case TXT_ST:
        {
          kb_value = key_read();
          if      (kb_value == ENTER_P)
           {
            lcd_init_text();
            lcd_clear();
            lcd_puts("        Roberto Fonseca Iannini         ",0,0);
            lcd_puts("________________________________________",0,1);
            lcd_puts("                                        ",0,2);
            lcd_puts("UP: text mode    ",10,5);
            lcd_puts("DOWN: graph mode   ",10,6);
            next_st = IDLE_ST;
           }
          else if (kb_value == CLR_P)
           {
            lcd_clear();
           }
          else if ( (kb_value == RIGHT_P) || (kb_value == LEFT_P) )
           {
            lcd_puts("<-- --> ",16,1);
           }
          else if ( (kb_value == UP_P) || (kb_value == DOWN_P) )
           {
            lcd_puts("UP/down ",16,20);
           };
         break;
        }
      case G_ST:
        {
          kb_value = key_read();
          if      (kb_value == ENTER_P)
           {
            lcd_init_text();
            lcd_clear();
            lcd_puts("        Roberto Fonseca Iannini         ",0,1);
            lcd_puts("________________________________________",0,2);
            lcd_puts("                                        ",0,3);
            lcd_puts("UP: text mode    ",10,5);
            lcd_puts("DOWN: graph mode   ",10,6);
            next_st = IDLE_ST;
           }
          else if (kb_value == CLR_P)
           {
            lcd_clear();
           }
          else if (kb_value == RIGHT_P)
           {
            while (print < 11)
              {
               lcd_graph_inv(0, 0, 240, 128);         // lets see the whole
               delay_ms(250);                         // image inverted!
               print++;
              }
            print = 0;
           }
          else if (kb_value == LEFT_P)
           {
            //lcd_bitmap_plot(0, 0, 240, 128, img5);  // super mae
            //lcd_bitmap_plot(0, 0, 240, 128, img6);  // Karime
            //delay_s(1);
            lcd_bitmap_plot(41, 0, 128, 64, img3);  // eita
            lcd_bitmap_plot(40, 64, 128, 64, img3);  // eita
            delay_s(1);
            lcd_bitmap_plot(0, 0, 31, 33, img2); //31 33
            lcd_bitmap_plot(1, 34, 31, 33, img2);
            lcd_bitmap_plot(2, 68, 31, 33, img2);
            lcd_bitmap_plot(3, 102, 31, 33, img2);
            delay_s(1);
            lcd_bitmap_plot(190, 0, 35, 33, img7); // 35 33
            lcd_bitmap_plot(191, 34, 35, 33, img7);
            lcd_bitmap_plot(192, 68, 35, 33, img7);
            lcd_bitmap_plot(196, 102, 35, 33, img7);
/*
            lcd_bitmap_plot(200, 0, 24, 24, img1); // 24 24
            lcd_bitmap_plot(204, 25, 24, 24, img1); // 24 24
            lcd_bitmap_plot(205, 50, 24, 24, img1); // 24 24
            lcd_bitmap_plot(206, 75, 24, 24, img1); // 24 24
            lcd_bitmap_plot(207, 100, 24, 24, img1); // 24 24
*/
            //lcd_bitmap_plot(208, 125, 24, 24, img1); // 24 24
            /*
            delay_s(1);
            lcd_bitmap_plot(64+3, 65, 9, 64, img3);
            delay_s(1);
            lcd_bitmap_plot(64+3+10, 66, 9, 64, img3);
            */
           }
          else if (kb_value == UP_P)
           {
            fsm_graph_demo_st();
           }
          else if (kb_value == DOWN_P)
           {
            lcd_bitmap_plot(0, 0, 240, 128, img4);
           };
          break;
        };
     } // switch
  }; // while
}; // main



//==============================================================================
// interrupt (high)
//==============================================================================
void interrupt
ISR(void)     //  The main ISR routine
{
  if (TMR0IE && TMR0IF)   // did the timer cause the interrupt?
  {
    TMR0ON = 0;                   // Timer0 On/Off Control bit
    //TMR0 = 0xF000; nao preciso pois o rollover sera em FF mesmo
    TMR0IF = 0;
/*
    if      K_RIGHT
     {
      L_GREEN = 1;
      L_RED   = 0;
      kb_value = RIGHT_P;
     }
    else if K_UP
     {
      L_GREEN = 1;
      L_RED   = 0;
      kb_value = UP_P;
   }
  else if K_DOWN
     {
      L_GREEN = 0;
      L_RED   = 1;
      kb_value = DOWN_P;
   }
  else if K_LEFT
     {
      L_GREEN = 0;
      L_RED   = 1;
      kb_value = LEFT_P;
   }
  else if K_ENTER
     {
      L_GREEN = 1;
      L_RED   = 1;
      kb_value = ENTER_P;
   }
  else if K_CLR
     {
      L_GREEN = 0;
      L_RED   = 0;
      kb_value = CLR_P;
   }
    else
     {
      kb_value = NONE_P;
      kb_hold =  0;
      if      (KB_OUT_A)
       {
        KB_OUT_A = 0;
        KB_OUT_B = 1;
        KB_OUT_C = 0;
       }
      else if (KB_OUT_B)
       {
        KB_OUT_A = 0;
        KB_OUT_B = 0;
        KB_OUT_C = 1;
       }
      else if (KB_OUT_C)
       {
        KB_OUT_A = 1;
        KB_OUT_B = 0;
        KB_OUT_C = 0;
       };
     };
    //KB_OUT_A = 0;
    //KB_OUT_B = 1;
    //KB_OUT_C = 0;
    //L_RED   = !L_RED;
    //L_GREEN = !L_GREEN;
*/
    TMR0ON = 1;                   // Timer0 On/Off Control bit
  }
  if (RBIE && RBIF)   // did something on PORTB change?
  {
    RBIF = 0;
  }
  if (RCIE && RCIF)   // did we receive serial I/O
  {
    bufferRX[indexRX] = RCREG;    // add to circular buffer
    indexRX++;
    indexRX &= 0x3F;          // 64-1 => %64
    RCIF = 0;
    serialRX = 1;
  };
}


//==============================================================================
// interrupt (high)
//==============================================================================
void interrupt low_priority
ISR_LOW(void)
{
}
